module Vlab
  
  class CommandManager
    def initialize
      @commands = {}
      register_command :fail
      register_command :help
      register_command :list
      register_command :login
    end

    def self.instance
      @command_manager ||= CommandManager.new
    end
    
    def register_command(command_obj)
      @commands[command_obj] = false
    end
  
    def [](command_name)
      command_name = command_name.intern
      return nil if @commands[command_name].nil?
      @commands[command_name] ||= load_and_instantiate(command_name)
    end
    
    def command_names
      @commands.keys.collect {|key| key.to_s}.sort
    end
    
    def run(args)
      cmd_name = args.shift.downcase

      cmd = find_command(cmd_name)

      Vlab::environment_manager.set(nil) if not cmd.local? 
      cmd.invoke(*args)
    end

    def find_command(cmd_name)
      self[cmd_name] || self["help"]
    end

    def load_and_instantiate(cmd_name)
      command_name = cmd_name.to_s
      retried = false

      begin
        const_name = command_name.capitalize.gsub(/_(.)/) { $1.upcase }
        Vlab::Commands.const_get("#{const_name}Command").new
      rescue NameError
        if retried then
          raise
        else
          retried = true
          require "vlab/commands/#{command_name}"
          retry
        end
      end
    end

  end
  
  module Commands; end
end
